/*
 * Copyright (c) 2001 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * The contents of this file constitute Original Code as defined in and
 * are subject to the Apple Public Source License Version 1.1 (the
 * "License").  You may not use this file except in compliance with the
 * License.  Please obtain a copy of the License at
 * http://www.apple.com/publicsource and read it before using this file.
 * 
 * This Original Code and all software distributed under the License are
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*	CFInternal.h
	Copyright 1998-2001, Apple, Inc. All rights reserved.
*/

/*
        NOT TO BE USED OUTSIDE CF!
*/

#if !defined(__COREFOUNDATION_CFINTERNAL__)
#define __COREFOUNDATION_CFINTERNAL__ 1

#if !defined(CF_BUILDING_CF)
#error The header file CFInternal.h is for the exclusive use of CoreFoundation. No other project should include it.
#endif

#include <CoreFoundation/CFBase.h>
#include <CoreFoundation/CFString.h>
#include <CoreFoundation/CFDate.h>
#include "CFRuntime.h"
#if defined(__MACH__) || defined(__LINUX__) || defined(__FREEBSD__)
#include <pthread.h>
#endif

#if defined(__LINUX__) || defined(__MACOS8__) || defined(__FREEBSD__)
#define __private_extern__
#endif


#if defined(DEBUG)
    #define __CFAssert(cond, prio, desc, a1, a2, a3, a4, a5)	\
	do {			\
	    if (!(cond)) {	\
		CFLog(prio, CFSTR(desc), a1, a2, a3, a4, a5); \
		__CFAssertAbort();	\
	    }			\
	} while (0)
#else
    #define __CFAssert(cond, prio, desc, a1, a2, a3, a4, a5)	\
	do {} while (0)
#endif

#define CFAssert(condition, priority, description)			\
    __CFAssert((condition), (priority), description, 0, 0, 0, 0, 0)
#define CFAssert1(condition, priority, description, a1)			\
    __CFAssert((condition), (priority), description, (a1), 0, 0, 0, 0)
#define CFAssert2(condition, priority, description, a1, a2)		\
    __CFAssert((condition), (priority), description, (a1), (a2), 0, 0, 0)
#define CFAssert3(condition, priority, description, a1, a2, a3)		\
    __CFAssert((condition), (priority), description, (a1), (a2), (a3), 0, 0)
#define CFAssert4(condition, priority, description, a1, a2, a3, a4)	\
    __CFAssert((condition), (priority), description, (a1), (a2), (a3), (a4), 0)

#define __kCFLogAssertion	15

#if defined(DEBUG)
extern void __CFGenericValidateType_(CFTypeRef cf, CFTypeID type, char *func);
#define __CFGenericValidateType(cf, type) __CFGenericValidateType_(cf, type, __PRETTY_FUNCTION__)
#else
#define __CFGenericValidateType(cf, type) 
#endif


/* Bit manipulation macros */
/* Bits are numbered from 31 on left to 0 on right */
/* May or may not work if you use them on bitfields in types other than UInt32, bitfields the full width of a UInt32, or anything else for which they were not designed. */
#define __CFBitfieldMask(N1, N2)	((((UInt32)~0UL) << (31UL - (N1) + (N2))) >> (31UL - N1))
#define __CFBitfieldGetValue(V, N1, N2)	(((V) & __CFBitfieldMask(N1, N2)) >> (N2))
#define __CFBitfieldSetValue(V, N1, N2, X)	((V) = ((V) & ~__CFBitfieldMask(N1, N2)) | (((X) << (N2)) & __CFBitfieldMask(N1, N2)))
#define __CFBitfieldMaxValue(N1, N2)	__CFBitfieldGetValue(0xFFFFFFFFUL, (N1), (N2))

#define __CFBitIsSet(V, N)  (((V) & (1UL << (N))) != 0)
#define __CFBitSet(V, N)  ((V) |= (1UL << (N)))
#define __CFBitClear(V, N)  ((V) &= ~(1UL << (N)))

typedef struct ___CFThreadSpecificData {
    void *_unused1;
    void *_allocator;
    void *_runLoop;
// If you add things to this struct, add cleanup to __CFFinalizeThreadData()
} __CFThreadSpecificData;

extern __CFThreadSpecificData *__CFGetThreadSpecificData(void);

#if defined(__MACH__) || defined(__LINUX__) || defined(__FREEBSD__)
extern pthread_key_t __CFTSDKey;
#endif
#if defined(__WIN32__)
extern DWORD __CFTSDKey;
#endif

CF_INLINE __CFThreadSpecificData *__CFGetThreadSpecificData_inline(void) {
#if defined(__MACOS8__)
    return __CFGetThreadSpecificData();
#elif defined(__MACH__) || defined(__LINUX__) || defined(__FREEBSD__)
    __CFThreadSpecificData *data = pthread_getspecific(__CFTSDKey);
    return data ? data : __CFGetThreadSpecificData();
#elif defined(__WIN32__)
    __CFThreadSpecificData *data = TlsGetValue(__CFTSDKey);
    return data ? data : __CFGetThreadSpecificData();
#endif
}

CF_INLINE CFAllocatorRef __CFGetDefaultAllocator(void) {
    CFAllocatorRef allocator = __CFGetThreadSpecificData_inline()->_allocator;
    if (NULL == allocator) {
	allocator = kCFAllocatorSystemDefault;
    }
    return allocator;
}

// Don't define a __CFGetCurrentRunLoop(), because even internal clients should go through the real one

#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
    #define __CFMin(A,B) ({__typeof__(A) __a = (A); __typeof__(B) __b = (B); __a < __b ? __a : __b; })
    #define __CFMax(A,B) ({__typeof__(A) __a = (A); __typeof__(B) __b = (B); __a < __b ? __b : __a; })
#else /* __GNUC__ */
    #define __CFMin(A,B) ((A) < (B) ? (A) : (B))
    #define __CFMax(A,B) ((A) > (B) ? (A) : (B))
#endif /* __GNUC__ */

/* Secret CFAllocator hint bits */
#define __kCFAllocatorTempMemory	0x2
#define __kCFAllocatorNoPointers	0x10
#define __kCFAllocatorDoNotRecordEvent	0x100

CF_EXPORT CFAllocatorRef _CFTemporaryMemoryAllocator(void);

extern SInt64 __CFTimeIntervalToTSR(CFTimeInterval ti);
extern CFTimeInterval __CFTSRToTimeInterval(SInt64 tsr);
extern SInt64 __CFAbsoluteTimeToTSR(CFAbsoluteTime at);
extern CFAbsoluteTime __CFTSRToAbsoluteTime(SInt64 tsr);

extern CFStringRef __CFCopyFormattingDescription(CFTypeRef cf, CFDictionaryRef formatOptions);

/* result is long long or int, depending on doLonglong
*/
extern Boolean __CFStringScanInteger(CFStringInlineBuffer *buf, CFDictionaryRef locale, SInt32 *indexPtr, Boolean doLonglong, void *result);
extern Boolean __CFStringScanDouble(CFStringInlineBuffer *buf, CFDictionaryRef locale, SInt32 *indexPtr, double *resultPtr); 
extern Boolean __CFStringScanHex(CFStringInlineBuffer *buf, SInt32 *indexPtr, unsigned *result);


#if defined(__MACH__)
#define __kCFCharacterSetDir "/System/Library/CoreServices"
#elif defined(__LINUX__) || defined(__FREEBSD__)
#define __kCFCharacterSetDir _CF_CHARSET_DIR_ /*"/usr/local/lib"*/
#endif


/* Buffer size for file pathname */
#if defined(__WIN32__)
    #define CFMaxPathSize ((CFIndex)262)
    #define CFMaxPathLength ((CFIndex)260)
#elif defined(__MACOS8__)
    #define CFMaxPathSize ((CFIndex)256)
    #define CFMaxPathLength ((CFIndex)255)
#else
    #define CFMaxPathSize ((CFIndex)1026)
    #define CFMaxPathLength ((CFIndex)1024)
#endif

#if defined(__MACH__)
extern bool __CFOASafe;
extern void __CFSetLastAllocationEventName(void *ptr, const char *classname);
#else
#define __CFOASafe 0
#define __CFSetLastAllocationEventName(a, b)
#endif


#if !defined(__MACOS8__)
CF_EXPORT CFStringRef _CFCreateLimitedUniqueString();
#endif !__MACOS8__

extern CFStringRef __CFCopyEthernetAddrString(void);

/* Comparators are passed the address of the values; this is somewhat different than CFComparatorFunction is used in public API usually. */
CF_EXPORT CFIndex	CFBSearch(const void *element, CFIndex elementSize, const void *list, CFIndex count, CFComparatorFunction comparator, void *context);

CF_EXPORT void		CFLog(int p, CFStringRef str, ...);

CF_EXPORT CFHashCode	CFHashBytes(UInt8 *bytes, CFIndex length);

CF_EXPORT CFStringEncoding CFStringFileSystemEncoding(void);

extern const void *__CFTypeCollectionRetain(CFAllocatorRef allocator, const void *ptr);
extern void __CFTypeCollectionRelease(CFAllocatorRef allocator, const void *ptr);

#if defined(__MACH__)
// In libSystem
extern void _spin_lock(UInt32 *lockp);
extern void _spin_unlock(UInt32 *lockp);

#define __CFSpinLock(A)		_spin_lock(A)
#define __CFSpinUnlock(A)	_spin_unlock(A)

#else

#define __CFSpinLock(A)		do {} while (0)
#define __CFSpinUnlock(A)	do {} while (0)

#endif

extern void __CFAbort(void);
extern void __CFAssertAbort(void);


#if defined(__svr4__) || defined(__hpux__)
#include <errno.h>
#elif defined(__WIN32__)
#elif defined(__MACOS8__)
#include <errno.h>
#elif defined(__MACH__) || defined(__LINUX__) || defined(__FREEBSD__)
#include <sys/errno.h>
#endif

#define thread_errno() errno
#define thread_set_errno(V) do {errno = (V);} while (0)

extern void *__CFStartSimpleThread(void *func, void *arg);


#if !defined(__MACH__)

#define CF_IS_OBJC(typeID, obj)	(false)

#define CF_OBJC_FUNCDISPATCH0(typeID, rettype, obj, sel)
#define CF_OBJC_FUNCDISPATCH1(typeID, rettype, obj, sel, a1)
#define CF_OBJC_FUNCDISPATCH2(typeID, rettype, obj, sel, a1, a2)
#define CF_OBJC_FUNCDISPATCH3(typeID, rettype, obj, sel, a1, a2, a3)
#define CF_OBJC_FUNCDISPATCH4(typeID, rettype, obj, sel, a1, a2, a3, a4)

#endif __MACH__

#if defined(__MACOS8__) || defined(__LINUX__) || defined(__FREEBSD__)
#define __CFISAForTypeID(x) (NULL)
#endif __MACOS8__

extern void __CFRuntimeSetInstanceISA(CFTypeRef cf, void *isa);

#if defined(__MACH__)

struct objc_class {     // nasty, nasty
        long __fields0__[2];
        const char *name;
        long __fields1__[7];
};

extern struct objc_class *__CFRuntimeObjCConstantStringClass;

CF_INLINE void *__CFISAForTypeID(CFTypeID typeID) {
    extern struct objc_class *__CFRuntimeObjCClassTable;
    return (void *)(__CFRuntimeObjCClassTable + typeID);
}

typedef void *SEL;
typedef void *id;

extern SEL (*__CFGetObjCSelector)(const char *);
extern id (*__CFSendObjCMsg)(id, SEL, ...);
extern id (*__CFGetObjCClass)(const char *);

#define CF_IS_CONSTANT_STRING(obj) (((CFRuntimeBase *)(obj))->_isa == __CFRuntimeObjCConstantStringClass)

// Although it might seem to make better performance to check for NULL
// first, doing the other check first is better.
#define CF_IS_OBJC(typeID, obj)	(((CFRuntimeBase *)(obj))->_isa != __CFISAForTypeID(typeID) && ((CFRuntimeBase *)(obj))->_isa != NULL)

#define CF_OBJC_FUNCDISPATCH0(typeID, rettype, obj, sel) \
	if (CF_IS_OBJC(typeID, obj)) \
	{rettype (*func)(id, SEL) = (void *)__CFSendObjCMsg; \
	static SEL s = NULL; if (!s) s = __CFGetObjCSelector(sel); \
	return func((id)obj, s);}
#define CF_OBJC_FUNCDISPATCH1(typeID, rettype, obj, sel, a1) \
	if (CF_IS_OBJC(typeID, obj)) \
	{rettype (*func)(id, SEL, ...) = (void *)__CFSendObjCMsg; \
	static SEL s = NULL; if (!s) s = __CFGetObjCSelector(sel); \
	return func((id)obj, s, (a1));}
#define CF_OBJC_FUNCDISPATCH2(typeID, rettype, obj, sel, a1, a2) \
	if (CF_IS_OBJC(typeID, obj)) \
	{rettype (*func)(id, SEL, ...) = (void *)__CFSendObjCMsg; \
	static SEL s = NULL; if (!s) s = __CFGetObjCSelector(sel); \
	return func((id)obj, s, (a1), (a2));}
#define CF_OBJC_FUNCDISPATCH3(typeID, rettype, obj, sel, a1, a2, a3) \
	if (CF_IS_OBJC(typeID, obj)) \
	{rettype (*func)(id, SEL, ...) = (void *)__CFSendObjCMsg; \
	static SEL s = NULL; if (!s) s = __CFGetObjCSelector(sel); \
	return func((id)obj, s, (a1), (a2), (a3));}
#define CF_OBJC_FUNCDISPATCH4(typeID, rettype, obj, sel, a1, a2, a3, a4) \
	if (CF_IS_OBJC(typeID, obj)) \
	{rettype (*func)(id, SEL, ...) = (void *)__CFSendObjCMsg; \
	static SEL s = NULL; if (!s) s = __CFGetObjCSelector(sel); \
	return func((id)obj, s, (a1), (a2), (a3), (a4));}

#endif __MACH__

/* See comments in CFBase.c
*/
#if defined(__ppc__) && defined(__MACH__)
extern void __CF_FAULT_CALLBACK(void **ptr);
/* WARNING: Do NOT make the following declaration a prototype!!! */
extern intptr_t __CF_INVOKE_CALLBACK();

#define FAULT_CALLBACK(V) __CF_FAULT_CALLBACK(V)
#define INVOKE_CALLBACK1(P, A) ({intptr_t (*__p__)() = (void *)(P); intptr_t __v__ = (0 == ((uint32_t)__p__ & 0x2)) ? __p__(A) : __CF_INVOKE_CALLBACK(P, A); __v__;})
#define INVOKE_CALLBACK2(P, A, B) ({intptr_t (*__p__)() = (void *)(P); intptr_t __v__ = (0 == ((uint32_t)__p__ & 0x2)) ? __p__(A, B) : __CF_INVOKE_CALLBACK(P, A, B); __v__;})
#define INVOKE_CALLBACK3(P, A, B, C) ({intptr_t (*__p__)() = (void *)(P); intptr_t __v__ = (0 == ((uint32_t)__p__ & 0x2)) ? __p__(A, B, C) : __CF_INVOKE_CALLBACK(P, A, B, C); __v__;})
#define INVOKE_CALLBACK4(P, A, B, C, D) ({intptr_t (*__p__)() = (void *)(P); intptr_t __v__ = (0 == ((uint32_t)__p__ & 0x2)) ? __p__(A, B, C, D) : __CF_INVOKE_CALLBACK(P, A, B, C, D); __v__;})
#define INVOKE_CALLBACK5(P, A, B, C, D, E) ({intptr_t (*__p__)() = (void *)(P); intptr_t __v__ = (0 == ((uint32_t)__p__ & 0x2)) ? __p__(A, B, C, D, E) : __CF_INVOKE_CALLBACK(P, A, B, C, D, E); __v__;})
#else
#define FAULT_CALLBACK(V)
#define INVOKE_CALLBACK1(P, A) (*(void *(*)())(P))(A)
#define INVOKE_CALLBACK2(P, A, B) (*(void *(*)())(P))(A, B)
#define INVOKE_CALLBACK3(P, A, B, C) (*(void *(*)())(P))(A, B, C)
#define INVOKE_CALLBACK4(P, A, B, C, D) (*(void *(*)())(P))(A, B, C, D)
#define INVOKE_CALLBACK5(P, A, B, C, D, E) (*(void *(*)())(P))(A, B, C, D, E)
#endif


#endif /* ! __COREFOUNDATION_CFINTERNAL__ */

