/*
 * Copyright (c) 2001 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * The contents of this file constitute Original Code as defined in and
 * are subject to the Apple Public Source License Version 1.1 (the
 * "License").  You may not use this file except in compliance with the
 * License.  Please obtain a copy of the License at
 * http://www.apple.com/publicsource and read it before using this file.
 * 
 * This Original Code and all software distributed under the License are
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*	CFCharacterSetPriv.h
	Copyright 1998-2001, Apple, Inc. All rights reserved.
*/

#if !defined(__COREFOUNDATION_CFCHARACTERSET_PRIV__)
#define __COREFOUNDATION_CFCHARACTERSET_PRIV__ 1

#include <CoreFoundation/CFCharacterSet.h>

#if defined(__cplusplus)
extern "C" {
#endif

/* The following declarations might be publicized (probably in Jaguar ??) */
#define kCFCharacterSetSurrogateHighShift (10)
#define kCFCharacterSetSurrogateBase (0x0010000UL)
#define kCFCharacterSetSurrogateHighStart (0xD800UL)
#define kCFCharacterSetSurrogateHighEnd (0xDBFFUL)
#define kCFCharacterSetSurrogateLowStart (0xDC00UL)
#define kCFCharacterSetSurrogateLowEnd (0xDFFFUL)

CF_INLINE Boolean CFCharacterSetIsSurrogateHighCharacter(UniChar character) {
    return ((character >= kCFCharacterSetSurrogateHighStart) && (character <= kCFCharacterSetSurrogateHighEnd) ? true : false);
}

CF_INLINE Boolean CFCharacterSetIsSurrogateLowCharacter(UniChar character) {
    return ((character >= kCFCharacterSetSurrogateLowStart) && (character <= kCFCharacterSetSurrogateLowEnd) ? true : false);
}

CF_INLINE UInt32 CFCharacterSetGetLongCharacterForSurrogatePair(UniChar surrogateHigh, UniChar surrogateLow) {
    return ((surrogateHigh - kCFCharacterSetSurrogateHighStart) << kCFCharacterSetSurrogateHighShift) + (surrogateLow - kCFCharacterSetSurrogateLowStart) + kCFCharacterSetSurrogateBase;
}

/* Check to see if UCS4 character theChar is in the chraracter set */
CF_EXPORT Boolean CFCharacterSetIsLongCharacterMember(CFCharacterSetRef theSet, UInt32 theChar);

/* Check to see if the character represented by the surrogate pair surrogateHigh & surrogateLow is in the chraracter set */
CF_EXPORT Boolean CFCharacterSetIsSurrogatePairMember(CFCharacterSetRef theSet, UniChar surrogateHigh, UniChar surrogateLow) ;

#if defined(__cplusplus)
}
#endif

#endif /* !__COREFOUNDATION_CFCHARACTERSET_PRIV__ */
